// -*- coding: iso-8859-1-unix -*-

#include "stubs/data_service.h"
#include "DataKeyWrapper.h"

#include <cstdio>
#include <string>
#include <algorithm>
#include <iterator>
#include <stdexcept>
#include <cstring>

namespace tecgraf { namespace openbus { namespace data_service {
namespace core { namespace v1_02 {

const char datakey_version[] = "v1_01";
const char datakey_key_delimiter = ';';

template <typename C, typename V, typename I = std::size_t>
struct index_iterator
{
  typedef index_iterator<C, V, I> self_type;

  typedef V value_type;
  typedef V* pointer;
  typedef V& reference;
  typedef std::ptrdiff_t difference_type;
  typedef std::forward_iterator_tag iterator_category;

  index_iterator() {}
  index_iterator(C& c, I i)
    : i(i), c(&c) {}

  V& operator*() const
  {
    return (*c)[i];
  }
  self_type operator++(int)
  {
    self_type tmp(*this);
    ++*this;
    return tmp;
  } 

  self_type& operator++()
  {
    ++i;
    return *this;
  }

  I i;
  C* c;
};

template <typename C, typename I>
bool operator==(index_iterator<C, I> const& lhs,
                index_iterator<C, I> const& rhs)
{
  return lhs.i == rhs.i;
}

template <typename C, typename I>
bool operator!=(index_iterator<C, I> const& lhs,
                index_iterator<C, I> const& rhs)
{
  return !(lhs == rhs);
}

DataKeyWrapper::DataKeyWrapper(DataKey const& key)
{
  typedef index_iterator<DataKey const, unsigned char const> iterator_type;
  iterator_type first (key, 0u), last (key, key.length());
  iterator_type current = std::find(first, last, datakey_key_delimiter);
  if(current == last)
    throw InvalidDataKey();
  else if(std::distance(first, current) != std::strlen(datakey_version)
          || !std::equal(first, current, datakey_version))
    throw InvalidDataKey();
  ++current;
  // DataKey has version v1_01
  iterator_type next = std::find(current, last, datakey_key_delimiter);
  if(next == last)
    throw InvalidDataKey();
  system_deployment_id.insert(system_deployment_id.end(), current, next);
  current = next;
  ++current;
  next = std::find(current, last, datakey_key_delimiter);
  if(next == last)
    throw InvalidDataKey();
  datasource_id.insert(datasource_id.end(), current, next);
  ++next;
  data_id.insert(data_id.end(), next, last);
}

DataKeyWrapper::DataKeyWrapper(std::string const & system_deployment_id
                               , std::string const& datasource_id
                               , std::string const& data_id)
  : system_deployment_id(system_deployment_id)
  , datasource_id(datasource_id)
  , data_id(data_id)
{
}

DataKeyWrapper::~DataKeyWrapper()
{
}

DataKey DataKeyWrapper::dataKey()
{
  DataKey key;
  key.length(std::strlen(datakey_version) + 1
             + system_deployment_id.size() + 1
             + datasource_id.size() + 1
             + data_id.size());
  typedef index_iterator<DataKey, unsigned char> iterator_type;
  iterator_type first (key, 0u), last (key, key.length());
  
  iterator_type current = std::copy(
    datakey_version , datakey_version + std::strlen(datakey_version) , first);
  *current++ = datakey_key_delimiter;
  current = std::copy(system_deployment_id.begin(), system_deployment_id.end(),
                      current);
  *current++ = datakey_key_delimiter;
  current = std::copy(datasource_id.begin(), datasource_id.end(), current);
  *current++ = datakey_key_delimiter;
  current = std::copy(data_id.begin(), data_id.end(), current);
  return key;
}

} } } } }
